@@ -6,9 +6,11 @@ import pprint
 from itertools import repeat
 from collections import namedtuple
 from collections.abc import Mapping
-from ast import parse, walk, literal_eval, Import, ImportFrom
+from ast import parse, walk, Import, ImportFrom
 
-ModNode = namedtuple('ModNode', ['name', 'pkgdeps', 'extdeps'])
+__version__ = '0.1.2'
+
+ModNode = namedtuple('ModNode', ['name', 'pkgdeps', 'extdeps', 'futures'])
 ModNode.__doc__ = """Module node for dependency graph.
 
 Attributes
@@ -19,6 +21,8 @@ pkgdeps : frozenset of str
     Module dependencies in the same package.
 extdeps : frozenset of str
     External module dependencies from outside of the package.
+futures : frozenset of str
+    Import directive names antecedent to 'from __future__ import'
 """
 
 
@@ -125,7 +129,7 @@ class GlobalNames(object):
             self.entry(name, lineno)
 
     def _add_importfrom(self, node):
-        pkg, _, _ = node.module.rpartition('.')
+        pkg, _ = resolve_package_module(node.module, self.pkg, node.level)
         if pkg == self.pkg:
             return
         lineno = node.lineno
@@ -164,14 +168,48 @@ class GlobalNames(object):
             self.add(child, istopnode=True)
 
 
+def module_is_package(module, pkg, level):
+    """Returns whether or not the module name refers to the package."""
+    if level == 0:
+        return module == pkg
+    elif level == 1:
+        return module is None
+    else:
+        return False
+
+
+def module_from_package(module, pkg, level):
+    """Returns whether or not a module is from the package."""
+    if level == 0:
+        return module.startswith(pkg + '.')
+    elif level == 1:
+        return True
+    else:
+        return False
+
+
+def resolve_package_module(module, pkg, level, default=None):
+    """Returns a 2-tuple of package and module name, even for relative
+    imports
+    """
+    if level == 0:
+        p, _, m = module.rpartition('.')
+    elif level == 1:
+        p = pkg
+        m = module or default
+    else:
+        p = m = None
+    return p, m
+
+
 def make_node(name, pkg, allowed, glbnames):
     """Makes a node by parsing a file and traversing its AST."""
     raw = SOURCES[pkg, name]
     tree = parse(raw, filename=name)
     # we only want to deal with global import statements
-    pkgdot = pkg + '.'
     pkgdeps = set()
     extdeps = set()
+    futures = set()
     glbnames.module = name
     for a in tree.body:
         glbnames.add(a, istopnode=True)
@@ -183,18 +221,19 @@ def make_node(name, pkg, allowed, glbnames):
                 else:
                     extdeps.add(n.name)
         elif isinstance(a, ImportFrom):
-            if a.module == pkg:
+            if module_is_package(a.module, pkg, a.level):
                 pkgdeps.update(n.name for n in a.names if n.name in allowed)
-            elif not a.module or a.module.startswith(pkgdot):
-                if a.module is None:
-                    p, dot, m = pkg, ".", a.names[0].name
-                else:
-                    p, dot, m = a.module.rpartition('.')
+            elif module_from_package(a.module, pkg, a.level):
+                p, m = resolve_package_module(a.module, pkg, a.level,
+                                              default=a.names[0].name)
                 if p == pkg and m in allowed:
                     pkgdeps.add(m)
                 else:
                     extdeps.add(a.module)
-    return ModNode(name, frozenset(pkgdeps), frozenset(extdeps))
+            elif a.module == '__future__':
+                futures.update(n.name for n in a.names)
+    return ModNode(name, frozenset(pkgdeps), frozenset(extdeps),
+                   frozenset(futures))
 
 
 def make_graph(pkg, exclude=None):
@@ -255,7 +294,8 @@ class _LazyModule(_ModuleType):
     @classmethod
     def load(cls, pkg, mod, asname=None):
         if mod in _modules:
-            return _modules[pkg]
+            key = pkg if asname is None else mod
+            return _modules[key]
         else:
             return cls(pkg, mod, asname)
 
@@ -280,6 +320,7 @@ class _LazyModule(_ModuleType):
 
 """
 
+
 def get_lineno(node, default=0):
     """Gets the lineno of a node or returns the default."""
     return getattr(node, 'lineno', default)
@@ -308,7 +349,6 @@ def format_lazy_import(names):
     lines = ''
     for _, name, asname in names:
         pkg, _, _ = name.partition('.')
-        target = asname or pkg
         if asname is None:
             line = '{pkg} = _LazyModule.load({pkg!r}, {mod!r})\n'
         else:
@@ -332,7 +372,6 @@ def format_from_import(names):
 
 def rewrite_imports(name, pkg, order, imps):
     """Rewrite the global imports in the file given the amalgamation."""
-    pkgdot = pkg + '.'
     raw = SOURCES[pkg, name]
     tree = parse(raw, filename=name)
     replacements = []  # list of (startline, stopline, str) tuples
@@ -356,27 +395,26 @@ def rewrite_imports(name, pkg, order, imps):
                     imps.add(imp)
                     keep.append(imp)
             if len(keep) == 0:
-                s = ', '.join(n.name for n in  a.names)
+                s = ', '.join(n.name for n in a.names)
                 s = '# amalgamated ' + s + '\n'
             else:
                 s = format_lazy_import(keep)
             replacements.append((start, stop, s))
         elif isinstance(a, ImportFrom):
-            if not a.module:
-                a.module = pkg
-                p, dot, m = pkg, ".", ""
-            else:
-                p, dot, m = a.module.rpartition('.')
-            if a.module == pkg:
+            p, m = resolve_package_module(a.module, pkg, a.level, default='')
+            if module_is_package(a.module, pkg, a.level):
                 for n in a.names:
                     if n.name in order:
                         msg = ('Cannot amalgamate import of '
                                'amalgamated module:\n\n  from {0} import {1}\n'
                                '\nin {0}/{2}.py').format(pkg, n.name, name)
                         raise RuntimeError(msg)
-            elif a.module.startswith(pkgdot) and p == pkg and m in order:
+            elif p == pkg and m in order:
                 replacements.append((start, stop,
-                                     '# amalgamated ' + a.module + '\n'))
+                                     '# amalgamated ' + p + '.' + m + '\n'))
+            elif a.module == '__future__':
+                replacements.append((start, stop,
+                                     '# amalgamated __future__ directive\n'))
             else:
                 keep = []
                 for n in a.names:
@@ -401,12 +439,23 @@ def rewrite_imports(name, pkg, order, imps):
     return ''.join(lines)
 
 
+def sorted_futures(graph):
+    """Returns a sorted, unique list of future imports."""
+    f = set()
+    for value in graph.values():
+        f |= value.futures
+    return sorted(f)
+
+
 def amalgamate(order, graph, pkg):
     """Create amalgamated source."""
     src = ('\"\"\"Amalgamation of {0} package, made up of the following '
            'modules, in order:\n\n* ').format(pkg)
     src += '\n* '.join(order)
     src += '\n\n\"\"\"\n'
+    futures = sorted_futures(graph)
+    if len(futures) > 0:
+        src += 'from __future__ import ' + ', '.join(futures) + '\n'
     src += LAZY_IMPORTS
     imps = set()
     for name in order:
