@@ -1,31 +1,33 @@
 # -*- coding: utf-8 -*-
 """Aliases for the xonsh shell."""
-
-from argparse import ArgumentParser, Action
-import builtins
-from collections.abc import MutableMapping, Iterable, Sequence
 import os
 import sys
 import shlex
+import inspect
+import argparse
+import builtins
+import collections.abc as abc
 
+from xonsh.lazyasd import lazyobject
 from xonsh.dirstack import cd, pushd, popd, dirs, _get_cwd
 from xonsh.environ import locate_binary
 from xonsh.foreign_shells import foreign_shell_data
-from xonsh.jobs import jobs, fg, bg, kill_all_jobs
-from xonsh.history import main as history_alias
-from xonsh.platform import ON_ANACONDA, ON_DARWIN, ON_WINDOWS, scandir
+from xonsh.jobs import jobs, fg, bg, clean_jobs
+from xonsh.history import history_main
+from xonsh.platform import (ON_ANACONDA, ON_DARWIN, ON_WINDOWS, ON_FREEBSD,
+                            scandir)
 from xonsh.proc import foreground
-from xonsh.replay import main as replay_main
+from xonsh.replay import replay_main
 from xonsh.timings import timeit_alias
 from xonsh.tools import (XonshError, argvquote, escape_windows_cmd_string,
                          to_bool)
-from xonsh.vox import Vox
-from xonsh.xontribs import main as xontribs_main
+from xonsh.xontribs import xontribs_main
 from xonsh.xoreutils import _which
-from xonsh.completers._aliases import completer_alias 
+
+import xonsh.completers._aliases as xca
 
 
-class Aliases(MutableMapping):
+class Aliases(abc.MutableMapping):
     """Represents a location to hold and look up aliases."""
 
     def __init__(self, *args, **kwargs):
@@ -43,7 +45,7 @@ class Aliases(MutableMapping):
         val = self._raw.get(key)
         if val is None:
             return default
-        elif isinstance(val, Iterable) or callable(val):
+        elif isinstance(val, abc.Iterable) or callable(val):
             return self.eval_alias(val, seen_tokens={key})
         else:
             msg = 'alias of {!r} has an inappropriate type: {!r}'
@@ -91,7 +93,8 @@ class Aliases(MutableMapping):
         builtin function and if alias is only a single command.
         """
         word = line.split(' ', 1)[0]
-        if word in builtins.aliases and isinstance(self.get(word), Sequence):
+        if word in builtins.aliases and isinstance(self.get(word),
+                                                   abc.Sequence):
             word_idx = line.find(word)
             expansion = ' '.join(self.get(word))
             line = line[:word_idx] + expansion + line[word_idx+len(word):]
@@ -141,24 +144,20 @@ class Aliases(MutableMapping):
                 p.pretty(dict(self))
 
 
-
-def exit(args, stdin=None):  # pylint:disable=redefined-builtin,W0622
+def xonsh_exit(args, stdin=None):
     """Sends signal to exit shell."""
+    if not clean_jobs():
+        # Do not exit if jobs not cleaned up
+        return None, None
     builtins.__xonsh_exit__ = True
-    kill_all_jobs()
     print()  # gimme a newline
     return None, None
 
 
-_SOURCE_FOREIGN_PARSER = None
-
-
-def _ensure_source_foreign_parser():
-    global _SOURCE_FOREIGN_PARSER
-    if _SOURCE_FOREIGN_PARSER is not None:
-        return _SOURCE_FOREIGN_PARSER
+@lazyobject
+def _SOURCE_FOREIGN_PARSER():
     desc = "Sources a file written in a foreign shell language."
-    parser = ArgumentParser('source-foreign', description=desc)
+    parser = argparse.ArgumentParser('source-foreign', description=desc)
     parser.add_argument('shell', help='Name or path to the foreign shell')
     parser.add_argument('files_or_code', nargs='+',
                         help='file paths to source or code in the target '
@@ -201,14 +200,12 @@ def _ensure_source_foreign_parser():
     parser.add_argument('--seterrpostcmd', default=None, dest='seterrpostcmd',
                         help='command(s) to set exit-on-error after all'
                              'other commands.')
-    _SOURCE_FOREIGN_PARSER = parser
     return parser
 
 
 def source_foreign(args, stdin=None):
     """Sources a file written in a foreign shell language."""
-    parser = _ensure_source_foreign_parser()
-    ns = parser.parse_args(args)
+    ns = _SOURCE_FOREIGN_PARSER.parse_args(args)
     if ns.prevcmd is not None:
         pass  # don't change prevcmd if given explicitly
     elif os.path.isfile(ns.files_or_code[0]):
@@ -254,11 +251,15 @@ def source_foreign(args, stdin=None):
 def source_alias(args, stdin=None):
     """Executes the contents of the provided files in the current context.
     If sourced file isn't found in cwd, search for file along $PATH to source
-    instead"""
+    instead.
+    """
+    env = builtins.__xonsh_env__
+    encoding = env.get('XONSH_ENCODING')
+    errors = env.get('XONSH_ENCODING_ERRORS')
     for fname in args:
         if not os.path.isfile(fname):
             fname = locate_binary(fname)
-        with open(fname, 'r') as fp:
+        with open(fname, 'r', encoding=encoding, errors=errors) as fp:
             builtins.execx(fp.read(), 'exec', builtins.__xonsh_ctx__)
 
 
@@ -279,38 +280,58 @@ def source_cmd(args, stdin=None):
     args.append('--envcmd=set')
     args.append('--seterrpostcmd=if errorlevel 1 exit 1')
     args.append('--use-tmpfile=1')
-    return source_foreign(args, stdin=stdin)
+    with builtins.__xonsh_env__.swap(PROMPT='$P$G'):
+        return source_foreign(args, stdin=stdin)
 
 
 def xexec(args, stdin=None):
-    """Replaces current process with command specified and passes in the
-    current xonsh environment.
+    """exec [-h|--help] command [args...]
+
+    exec (also aliased as xexec) uses the os.execvpe() function to
+    replace the xonsh process with the specified program. This provides
+    the functionality of the bash 'exec' builtin::
+
+        >>> exec bash -l -i
+        bash $
+
+    The '-h' and '--help' options print this message and exit.
+
+    Notes
+    -----
+    This command **is not** the same as the Python builtin function
+    exec(). That function is for running Python code. This command,
+    which shares the same name as the sh-lang statement, is for launching
+    a command directly in the same process. In the event of a name conflict,
+    please use the xexec command directly or dive into subprocess mode
+    explicitly with ![exec command]. For more details, please see
+    http://xon.sh/faq.html#exec.
     """
-    env = builtins.__xonsh_env__
-    denv = env.detype()
-    if len(args) > 0:
+    if len(args) == 0:
+        return (None, 'xonsh: exec: no args specified\n', 1)
+    elif args[0] == '-h' or args[0] == '--help':
+        return inspect.getdoc(xexec)
+    else:
+        denv = builtins.__xonsh_env__.detype()
         try:
             os.execvpe(args[0], args, denv)
         except FileNotFoundError as e:
             return (None, 'xonsh: exec: file not found: {}: {}'
                           '\n'.format(e.args[1], args[0]), 1)
-    else:
-        return (None, 'xonsh: exec: no args specified\n', 1)
 
 
-_BANG_N_PARSER = None
+@lazyobject
+def _BANG_N_PARSER():
+    parser = argparse.ArgumentParser(
+        '!n', usage='!n <n>',
+        description="Re-runs the nth command as specified in the argument.")
+    parser.add_argument('n', type=int, help='the command to rerun, may be '
+                                            'negative')
+    return parser
 
 
 def bang_n(args, stdin=None):
     """Re-runs the nth command as specified in the argument."""
-    global _BANG_N_PARSER
-    if _BANG_N_PARSER is None:
-        parser = _BANG_N_PARSER = ArgumentParser('!n', usage='!n <n>',
-                    description="Re-runs the nth command as specified in the argument.")
-        parser.add_argument('n', type=int, help='the command to rerun, may be negative')
-    else:
-        parser = _BANG_N_PARSER
-    ns = parser.parse_args(args)
+    ns = _BANG_N_PARSER.parse_args(args)
     hist = builtins.__xonsh_history__
     nhist = len(hist)
     n = nhist + ns.n if ns.n < 0 else ns.n
@@ -327,8 +348,9 @@ def bang_bang(args, stdin=None):
     return bang_n(['-1'])
 
 
-class AWitchAWitch(Action):
+class AWitchAWitch(argparse.Action):
     SUPPRESS = '==SUPPRESS=='
+
     def __init__(self, option_strings, version=None, dest=SUPPRESS,
                  default=SUPPRESS, **kwargs):
         super().__init__(option_strings=option_strings, dest=dest,
@@ -336,7 +358,7 @@ class AWitchAWitch(Action):
 
     def __call__(self, parser, namespace, values, option_string=None):
         import webbrowser
-        webbrowser.open('https://github.com/scopatz/xonsh/commit/f49b400')
+        webbrowser.open('https://github.com/xonsh/xonsh/commit/f49b400')
         parser.exit()
 
 
@@ -348,10 +370,10 @@ def which(args, stdin=None, stdout=None, stderr=None):
     `which` match.
     """
     desc = "Parses arguments to which wrapper"
-    parser = ArgumentParser('which', description=desc)
+    parser = argparse.ArgumentParser('which', description=desc)
     parser.add_argument('args', type=str, nargs='+',
                         help='The executables or aliases to search for')
-    parser.add_argument('-a','--all', action='store_true', dest='all',
+    parser.add_argument('-a', '--all', action='store_true', dest='all',
                         help='Show all matches in $PATH and xonsh.aliases')
     parser.add_argument('-s', '--skip-alias', action='store_true',
                         help='Do not search in xonsh.aliases', dest='skip')
@@ -390,7 +412,7 @@ def which(args, stdin=None, stdout=None, stderr=None):
         if pargs.exts:
             exts = pargs.exts
         else:
-            exts = builtins.__xonsh_env__.get('PATHEXT', ['.COM', '.EXE', '.BAT'])
+            exts = builtins.__xonsh_env__['PATHEXT']
     else:
         exts = None
 
@@ -400,12 +422,14 @@ def which(args, stdin=None, stdout=None, stderr=None):
         # skip alias check if user asks to skip
         if (arg in builtins.aliases and not pargs.skip):
             if pargs.plain or not pargs.verbose:
-                if isinstance(builtins.aliases[arg], list):
+                if not callable(builtins.aliases[arg]):
                     print(' '.join(builtins.aliases[arg]), file=stdout)
                 else:
                     print(arg, file=stdout)
             else:
                 print("aliases['{}'] = {}".format(arg, builtins.aliases[arg]), file=stdout)
+                if callable(builtins.aliases[arg]):
+                    builtins.__xonsh_superhelp__(builtins.aliases[arg])
             nmatches += 1
             if not pargs.all:
                 continue
@@ -446,21 +470,18 @@ def which(args, stdin=None, stdout=None, stderr=None):
 
 def xonfig(args, stdin=None):
     """Runs the xonsh configuration utility."""
-    from xonsh.xonfig import main  # lazy import
-    return main(args)
+    from xonsh.xonfig import xonfig_main  # lazy import
+    return xonfig_main(args)
 
 
 @foreground
 def trace(args, stdin=None):
     """Runs the xonsh tracer utility."""
-    from xonsh.tracer import main  # lazy import
-    return main(args)
-
-
-def vox(args, stdin=None):
-    """Runs Vox environment manager."""
-    vox = Vox()
-    return vox(args, stdin=stdin)
+    from xonsh.tracer import tracermain  # lazy import
+    try:
+        return tracermain(args)
+    except SystemExit:
+        pass
 
 
 def showcmd(args, stdin=None):
@@ -493,16 +514,17 @@ def make_default_aliases():
         'jobs': jobs,
         'fg': fg,
         'bg': bg,
-        'EOF': exit,
-        'exit': exit,
-        'quit': exit,
+        'EOF': xonsh_exit,
+        'exit': xonsh_exit,
+        'quit': xonsh_exit,
+        'exec': xexec,
         'xexec': xexec,
         'source': source_alias,
         'source-zsh': ['source-foreign', 'zsh', '--sourcer=source'],
         'source-bash':  ['source-foreign', 'bash', '--sourcer=source'],
         'source-cmd': source_cmd,
         'source-foreign': source_foreign,
-        'history': history_alias,
+        'history': history_main,
         'replay': replay_main,
         '!!': bang_bang,
         '!n': bang_n,
@@ -512,10 +534,9 @@ def make_default_aliases():
         'scp-resume': ['rsync', '--partial', '-h', '--progress', '--rsh=ssh'],
         'showcmd': showcmd,
         'ipynb': ['jupyter', 'notebook', '--no-browser'],
-        'vox': vox,
         'which': which,
         'xontrib': xontribs_main,
-        'completer': completer_alias
+        'completer': xca.completer_alias
     }
     if ON_WINDOWS:
         # Borrow builtin commands from cmd.exe.
@@ -542,17 +563,10 @@ def make_default_aliases():
         default_aliases['call'] = ['source-cmd']
         default_aliases['source-bat'] = ['source-cmd']
         default_aliases['clear'] = 'cls'
-        # Add aliases specific to the Anaconda python distribution.
         if ON_ANACONDA:
-            def source_cmd_keep_prompt(args, stdin=None):
-                p = builtins.__xonsh_env__.get('PROMPT')
-                source_cmd(args, stdin=stdin)
-                builtins.__xonsh_env__['PROMPT'] = p
-            default_aliases['source-cmd-keep-promt'] = source_cmd_keep_prompt
-            default_aliases['activate'] = ['source-cmd-keep-promt',
-                                           'activate.bat']
-            default_aliases['deactivate'] = ['source-cmd-keep-promt',
-                                             'deactivate.bat']
+            # Add aliases specific to the Anaconda python distribution.
+            default_aliases['activate'] = ['source-cmd', 'activate.bat']
+            default_aliases['deactivate'] = ['source-cmd', 'deactivate.bat']
         if not locate_binary('sudo'):
             import xonsh.winutils as winutils
 
@@ -574,7 +588,14 @@ def make_default_aliases():
             default_aliases['sudo'] = sudo
     elif ON_DARWIN:
         default_aliases['ls'] = ['ls', '-G']
+    elif ON_FREEBSD:
+        default_aliases['grep'] = ['grep', '--color=auto']
+        default_aliases['egrep'] = ['egrep', '--color=auto']
+        default_aliases['fgrep'] = ['fgrep', '--color=auto']
+        default_aliases['ls'] = ['ls', '-G']
     else:
         default_aliases['grep'] = ['grep', '--color=auto']
+        default_aliases['egrep'] = ['egrep', '--color=auto']
+        default_aliases['fgrep'] = ['fgrep', '--color=auto']
         default_aliases['ls'] = ['ls', '--color=auto', '-v']
     return default_aliases
