@@ -7,7 +7,7 @@ import time
 import builtins
 
 from xonsh.tools import (XonshError, escape_windows_cmd_string, print_exception,
-    DefaultNotGiven)
+                         DefaultNotGiven)
 from xonsh.platform import HAS_PYGMENTS, ON_WINDOWS
 from xonsh.codecache import (should_use_cache, code_cache_name,
                              code_cache_check, get_cache_filename,
@@ -23,6 +23,7 @@ class _TeeOut(object):
         self.buffer = buf
         self.stdout = sys.stdout
         self.encoding = self.stdout.encoding
+        self.errors = self.stdout.errors
         sys.stdout = self
 
     def __del__(self):
@@ -34,7 +35,7 @@ class _TeeOut(object):
 
     def write(self, data):
         """Writes data to the original stdout and the buffer."""
-        #data = data.replace('\001', '').replace('\002', '')
+        # data = data.replace('\001', '').replace('\002', '')
         self.stdout.write(data)
         self.buffer.write(data)
 
@@ -55,6 +56,7 @@ class _TeeErr(object):
         self.buffer = buf
         self.stderr = sys.stderr
         self.encoding = self.stderr.encoding
+        self.errors = self.stderr.errors
         sys.stderr = self
 
     def __del__(self):
@@ -66,7 +68,7 @@ class _TeeErr(object):
 
     def write(self, data):
         """Writes data to the original stderr and the buffer."""
-        #data = data.replace('\001', '').replace('\002', '')
+        # data = data.replace('\001', '').replace('\002', '')
         self.stderr.write(data)
         self.buffer.write(data)
 
@@ -226,7 +228,6 @@ class BaseShell(object):
 
     def settitle(self):
         """Sets terminal title."""
-        _ = self
         env = builtins.__xonsh_env__  # pylint: disable=no-member
         term = env.get('TERM', None)
         # Shells running in emacs sets TERM to "dumb" or "eterm-color".
@@ -245,7 +246,7 @@ class BaseShell(object):
             with open(1, 'wb', closefd=False) as f:
                 # prevent xonsh from answering interative questions
                 # on the next command by writing the title
-                f.write("\x1b]2;{0}\x07".format(t).encode())
+                f.write("\x1b]0;{0}\x07".format(t).encode())
                 f.flush()
 
     @property
@@ -270,7 +271,6 @@ class BaseShell(object):
 
     def _append_history(self, tee_out=None, **info):
         """Append information about the command to the history."""
-        _ = self
         hist = builtins.__xonsh_history__  # pylint: disable=no-member
         info['rtn'] = hist.last_cmd_rtn
         tee_out = tee_out or None
