@@ -2,20 +2,23 @@
 """Implements the xonsh history object."""
 import os
 import sys
-import argparse
-import functools
-import operator
-import uuid
+import glob
+import json
 import time
-import datetime
+import uuid
+import argparse
 import builtins
-from glob import iglob
-from collections import deque, Sequence, OrderedDict
-from threading import Thread, Condition
+import datetime
+import functools
+import itertools
+import threading
+import collections
+import collections.abc as abc
 
+from xonsh.lazyasd import lazyobject
 from xonsh.lazyjson import LazyJSON, ljdump, LJNode
-from xonsh.tools import (ensure_int_or_slice, to_history_tuple,
-                         expanduser_abs_path)
+from xonsh.tools import (ensure_slice, to_history_tuple,
+                         expanduser_abs_path, ensure_timestamp)
 from xonsh.diff_history import _dh_create_parser, _dh_main_action
 
 
@@ -68,7 +71,7 @@ def _gc_bytes_to_rmfiles(hsize, files):
     return rmfiles
 
 
-class HistoryGC(Thread):
+class HistoryGC(threading.Thread):
     """Shell history garbage collection."""
 
     def __init__(self, wait_for_shell=True, size=None, *args, **kwargs):
@@ -112,12 +115,11 @@ class HistoryGC(Thread):
         This is sorted by the last closed time. Returns a list of (timestamp,
         file) tuples.
         """
-        _ = self  # this could be a function but is intimate to this class
         # pylint: disable=no-member
         xdd = builtins.__xonsh_env__.get('XONSH_DATA_DIR')
         xdd = expanduser_abs_path(xdd)
 
-        fs = [f for f in iglob(os.path.join(xdd, 'xonsh-*.json'))]
+        fs = [f for f in glob.iglob(os.path.join(xdd, 'xonsh-*.json'))]
         files = []
         for f in fs:
             try:
@@ -135,7 +137,7 @@ class HistoryGC(Thread):
         return files
 
 
-class HistoryFlusher(Thread):
+class HistoryFlusher(threading.Thread):
     """Flush shell history to disk periodically."""
 
     def __init__(self, filename, buffer, queue, cond, at_exit=False, *args,
@@ -176,7 +178,7 @@ class HistoryFlusher(Thread):
             ljdump(hist, f, sort_keys=True)
 
 
-class CommandField(Sequence):
+class CommandField(abc.Sequence):
     """A field in the 'cmds' portion of history."""
 
     def __init__(self, field, hist, default=None):
@@ -234,39 +236,29 @@ class CommandField(Sequence):
         return self is self.hist._queue[0]
 
 
-def _find_histfile_var(file_list=None, default=None):
-    if file_list is None:
-        return None
-    hist_file = None
-
-    found_hist = False
+def _find_histfile_var(file_list, default=None):
+    """Return the path of the history file
+    from the value of the envvar HISTFILE.
+    """
     for f in file_list:
         f = expanduser_abs_path(f)
         if not os.path.isfile(f):
             continue
         with open(f, 'r') as rc_file:
             for line in rc_file:
-                if "HISTFILE=" in line:
-                    evar = line.split(' ', 1)[-1]
-                    hist_file = evar.split('=', 1)[-1]
-                    for char in ['"', "'", '\n']:
-                        hist_file = hist_file.replace(char, '')
+                if line.startswith('HISTFILE='):
+                    hist_file = line.split('=', 1)[1].strip('\'"\n')
                     hist_file = expanduser_abs_path(hist_file)
                     if os.path.isfile(hist_file):
-                        found_hist = True
-                        break
-        if found_hist:
-            break
-
-    if hist_file is None:
-        default = expanduser_abs_path(default)
-        if os.path.isfile(default):
-            hist_file = default
-
-    return hist_file
+                        return hist_file
+    else:
+        if default:
+            default = expanduser_abs_path(default)
+            if os.path.isfile(default):
+                return default
 
 
-def _all_xonsh_parser(*args):
+def _all_xonsh_parser(**kwargs):
     """
     Returns all history as found in XONSH_DATA_DIR.
 
@@ -277,50 +269,42 @@ def _all_xonsh_parser(*args):
 
     files = [os.path.join(data_dir, f) for f in os.listdir(data_dir)
              if f.startswith('xonsh-') and f.endswith('.json')]
-    file_hist = []
+    ind = 0
     for f in files:
         try:
             json_file = LazyJSON(f, reopen=False)
-            file_hist.append(json_file.load()['cmds'])
         except ValueError:
             # Invalid json file
             pass
-    commands = [(c['inp'][:-1] if c['inp'].endswith('\n') else c['inp'],
-                 c['ts'][0])
-                for commands in file_hist for c in commands if c]
-    commands.sort(key=operator.itemgetter(1))
-    return [(c, t, ind) for ind, (c, t) in enumerate(commands)]
+        commands = json_file.load()['cmds']
+        for c in commands:
+            yield (c['inp'].rstrip(), c['ts'][0], ind)
+            ind += 1
 
 
-def _curr_session_parser(hist=None):
+def _curr_session_parser(hist=None, **kwargs):
     """
     Take in History object and return command list tuple with
     format: (name, start_time, index)
     """
     if hist is None:
         hist = builtins.__xonsh_history__
-    if not hist:
-        return None
-    start_times = [start for start, end in hist.tss]
-    names = [name[:-1] if name.endswith('\n') else name
-             for name in hist.inps]
-    commands = enumerate(zip(names, start_times))
-    return [(c, t, ind) for ind, (c, t) in commands]
+    start_times = (start for start, end in hist.tss)
+    names = (name.rstrip() for name in hist.inps)
+    for ind, (c, t) in enumerate(zip(names, start_times)):
+        yield (c, t, ind)
 
 
-def _zsh_hist_parser(location=None):
-    default_location = os.path.join('~', '.zsh_history')
-    location_list = [os.path.join('~', '.zshrc'),
-                     os.path.join('~', '.zprofile')]
+def _zsh_hist_parser(location=None, **kwargs):
+    """Yield commands from zsh history file"""
     if location is None:
-        location = _find_histfile_var(location_list, default_location)
-    z_hist_formatted = []
-    if os.path.isfile(location):
-        with open(location, 'r', errors='backslashreplace') as z_file:
-            z_txt = z_file.read()
-            z_hist = z_txt.splitlines()
-            if z_hist:
-                for ind, line in enumerate(z_hist):
+        location = _find_histfile_var([os.path.join('~', '.zshrc'),
+                                       os.path.join('~', '.zprofile')],
+                                      os.path.join('~', '.zsh_history'))
+    if location:
+        with open(location, 'r', errors='backslashreplace') as zsh_hist:
+            for ind, line in enumerate(zsh_hist):
+                if line.startswith(':'):
                     try:
                         start_time, command = line.split(';', 1)
                     except ValueError:
@@ -329,35 +313,27 @@ def _zsh_hist_parser(location=None):
                     try:
                         start_time = float(start_time.split(':')[1])
                     except ValueError:
-                        start_time = -1
-                    z_hist_formatted.append((command, start_time, ind))
-                return z_hist_formatted
+                        start_time = 0.0
+                    yield (command.rstrip(), start_time, ind)
+                else:
+                    yield (line.rstrip(), 0.0, ind)
 
     else:
-        print("No zsh history file found at: {}".format(location),
-              file=sys.stderr)
+        print("No zsh history file found", file=sys.stderr)
 
 
-def _bash_hist_parser(location=None):
-    default_location = os.path.join('~', '.bash_history')
-    location_list = [os.path.join('~', '.bashrc'),
-                     os.path.join('~', '.bash_profile')]
+def _bash_hist_parser(location=None, **kwargs):
+    """Yield commands from bash history file"""
     if location is None:
-        location = _find_histfile_var(location_list, default_location)
-    bash_hist_formatted = []
-    if os.path.isfile(location):
-        with open(location, 'r', errors='backslashreplace') as bash_file:
-            b_txt = bash_file.read()
-            bash_hist = b_txt.splitlines()
-            if bash_hist:
-                for ind, command in enumerate(bash_hist):
-                    bash_hist_formatted.append((command, 0.0, ind))
-                return bash_hist_formatted
+        location = _find_histfile_var([os.path.join('~', '.bashrc'),
+                                       os.path.join('~', '.bash_profile')],
+                                      os.path.join('~', '.bash_history'))
+    if location:
+        with open(location, 'r', errors='backslashreplace') as bash_hist:
+            for ind, line in enumerate(bash_hist):
+                yield (line.rstrip(), 0.0, ind)
     else:
-        import ipdb
-        ipdb.set_trace()
-        print("No bash history file found at: {}".format(location),
-              file=sys.stderr)
+        print("No bash history file", file=sys.stderr)
 
 
 @functools.lru_cache()
@@ -367,43 +343,24 @@ def _hist_create_parser():
                                 description='Tools for dealing with history')
     subp = p.add_subparsers(title='action', dest='action')
     # session action
-    show = subp.add_parser('show', aliases=['session'],
+    show = subp.add_parser('show', prefix_chars='-+',
                            help='displays session history, default action')
     show.add_argument('-r', dest='reverse', default=False,
-                      action='store_true',
-                      help='reverses the direction')
-    show.add_argument('n', nargs='?', default=None,
-                      help='display n\'th history entry if n is a simple '
-                           'int, or range of entries if it is Python '
-                           'slice notation')
-    # all action
-    xonsh = subp.add_parser('xonsh', aliases=['all'],
-                            help='displays history from all sessions')
-    xonsh.add_argument('-r', dest='reverse', default=False,
-                       action='store_true',
-                       help='reverses the direction')
-    xonsh.add_argument('n', nargs='?', default=None,
-                       help='display n\'th history entry if n is a '
-                            'simple int, or range of entries if it '
-                            'is Python slice notation')
-    # zsh action
-    zsh = subp.add_parser('zsh', help='displays history from zsh sessions')
-    zsh.add_argument('-r', dest='reverse', default=False,
-                     action='store_true',
-                     help='reverses the direction')
-    zsh.add_argument('n', nargs='?', default=None,
-                     help='display n\'th history entry if n is a '
-                     'simple int, or range of entries if it '
-                     'is Python slice notation')
-    # bash action
-    bash = subp.add_parser('bash', help='displays history from bash sessions')
-    bash.add_argument('-r', dest='reverse', default=False,
-                      action='store_true',
-                      help='reverses the direction')
-    bash.add_argument('n', nargs='?', default=None,
-                      help='display n\'th history entry if n is a '
-                      'simple int, or range of entries if it '
-                      'is Python slice notation')
+                      action='store_true', help='reverses the direction')
+    show.add_argument('-n', dest='numerate', default=False, action='store_true',
+                      help='numerate each command')
+    show.add_argument('-t', dest='timestamp', default=False,
+                      action='store_true', help='show command timestamps')
+    show.add_argument('-T', dest='end_time', default=None,
+                      help='show only commands before timestamp')
+    show.add_argument('+T', dest='start_time', default=None,
+                      help='show only commands after timestamp')
+    show.add_argument('-f', dest='datetime_format', default=None,
+                      help='the datetime format to be used for filtering and printing')
+    show.add_argument('session', nargs='?', choices=_HIST_SESSIONS.keys(), default='session',
+                      help='Choose a history session, defaults to current session')
+    show.add_argument('slices', nargs='*', default=None,
+                      help='display history entries or range of entries')
     # 'id' subcommand
     subp.add_parser('id', help='displays the current session id')
     # 'file' subcommand
@@ -437,104 +394,114 @@ def _hist_create_parser():
     return p
 
 
-def _show(ns=None, hist=None, start_index=None, end_index=None,
-          start_time=None, end_time=None, location=None):
-    """
-    Show the requested portion of shell history.
-    Accepts multiple history sources (xonsh, bash, zsh)
-
-    May be invoked as an alias with `history all/bash/zsh` which will
-    provide history as stdout or with `__xonsh_history__.show()`
-    which will return the history as a list with each item
-    in the tuple form (name, start_time, index).
-
-    If invoked via __xonsh_history__.show() then the ns parameter
-    can be supplied as a str with the follow options:
-        `session` - returns xonsh history from current session
-        `all`     - returns xonsh history from all sessions
-        `zsh`     - returns all zsh history
-        `bash`    - returns all bash history
-    """
-    # Check if ns is a string, meaning it was invoked from
-    # __xonsh_history__
-    alias = True
-    valid_formats = {'session': functools.partial(_curr_session_parser, hist),
-                     'show': functools.partial(_curr_session_parser, hist),
-                     'all': _all_xonsh_parser,
-                     'xonsh': _all_xonsh_parser,
-                     'zsh': functools.partial(_zsh_hist_parser, location),
-                     'bash': functools.partial(_bash_hist_parser, location)}
-    if isinstance(ns, str) and ns in valid_formats.keys():
-        ns = _hist_create_parser().parse_args([ns])
-        alias = False
-    if not ns:
-        ns = _hist_create_parser().parse_args(['all'])
-        alias = False
-    try:
-        commands = valid_formats[ns.action]()
-    except KeyError:
-        print("{} is not a valid history format".format(ns.action))
-        return None
-    if not commands:
-        return None
-    if start_time:
-        if isinstance(start_time, datetime.datetime):
-            start_time = start_time.timestamp()
-        if isinstance(start_time, float):
-            commands = [c for c in commands if c[1] >= start_time]
-        else:
-            print("Invalid start time, must be float or datetime.")
-    if end_time:
-        if isinstance(end_time, datetime.datetime):
-            end_time = end_time.timestamp()
-        if isinstance(end_time, float):
-            commands = [c for c in commands if c[1] <= end_time]
-        else:
-            print("Invalid end time, must be float or datetime.")
-    idx = None
-    if ns:
-        idx = ensure_int_or_slice(ns.n)
-        if idx is False:
-            print("{} is not a valid input.".format(ns.n),
-                  file=sys.stderr)
+def _hist_get_portion(commands, slices):
+    """Yield from portions of history commands."""
+    if len(slices) == 1:
+        s = slices[0]
+        try:
+            yield from itertools.islice(commands, s.start, s.stop, s.step)
             return
-        elif isinstance(idx, int):
-            try:
-                commands = [commands[idx]]
-            except IndexError:
-                err = "Index likely not in range. Only {} commands."
-                print(err.format(len(commands)))
-                return
-    else:
-        idx = slice(start_index, end_index)
+        except ValueError:  # islice failed
+            pass
+    commands = list(commands)
+    for s in slices:
+        yield from commands[s]
 
-    if (isinstance(idx, slice) and
-            start_time is None and end_time is None):
-        commands = commands[idx]
 
-    if ns and ns.reverse:
-        commands = list(reversed(commands))
+def _hist_filter_ts(commands, start_time, end_time):
+    """Yield only the commands between start and end time."""
+    for cmd in commands:
+        if start_time <= cmd[1] < end_time:
+            yield cmd
 
-    if commands:
-        digits = len(str(max([i for c, t, i in commands])))
-        if alias:
-            for c, t, i in commands:
-                for line_ind, line in enumerate(c.split('\n')):
-                    if line_ind == 0:
-                        print('{:>{width}}: {}'.format(i, line,
-                                                       width=digits + 1))
-                    else:
-                        print(' {:>>{width}} {}'.format('', line,
-                                                        width=digits + 1))
+
+def _hist_get(session='session', *, slices=None, datetime_format=None,
+              start_time=None, end_time=None, location=None):
+    """Get the requested portion of shell history.
+
+    Parameters
+    ----------
+    session: {'session', 'all', 'xonsh', 'bash', 'zsh'}
+        The history session to get.
+    slices : list of slice-like objects, optional
+        Get only portions of history.
+    start_time, end_time: float, optional
+        Filter commands by timestamp.
+    location: string, optional
+        The history file location (bash or zsh)
+
+    Returns
+    -------
+    generator
+       A filtered list of commands
+    """
+    cmds = _HIST_SESSIONS[session](location=location)
+    if slices:
+        # transform/check all slices
+        slices = [ensure_slice(s) for s in slices]
+        cmds = _hist_get_portion(cmds, slices)
+    if start_time or end_time:
+        if start_time is None:
+            start_time = 0.0
         else:
-            return commands
+            start_time = ensure_timestamp(start_time, datetime_format)
+        if end_time is None:
+            end_time = float('inf')
+        else:
+            end_time = ensure_timestamp(end_time, datetime_format)
+        cmds = _hist_filter_ts(cmds, start_time, end_time)
+    return cmds
+
+
+def _hist_show(ns, *args, **kwargs):
+    """Show the requested portion of shell history.
+    Accepts same parameters with `_hist_get`.
+    """
+    try:
+        commands = _hist_get(ns.session,
+                             slices=ns.slices,
+                             start_time=ns.start_time,
+                             end_time=ns.end_time,
+                             datetime_format=ns.datetime_format)
+    except ValueError as err:
+        print("history: error: {}".format(err), file=sys.stderr)
+        return
+    if ns.reverse:
+        commands = reversed(list(commands))
+    if not ns.numerate and not ns.timestamp:
+        for c, _, _ in commands:
+            print(c)
+    elif not ns.timestamp:
+        for c, _, i in commands:
+            print('{}: {}'.format(i, c))
+    elif not ns.numerate:
+        for c, ts, _ in commands:
+            dt = datetime.datetime.fromtimestamp(ts).ctime()
+            print('({}) {}'.format(dt, c))
+    else:
+        for c, ts, i in commands:
+            dt = datetime.datetime.fromtimestamp(ts).ctime()
+            print('{}:({}) {}'.format(i, dt, c))
 
 
-#
 # Interface to History
-#
 class History(object):
-    """Xonsh session history."""
+    """Xonsh session history.
+
+    Attributes
+    ----------
+    rtns : sequence of ints
+        The return of the command (ie, 0 on success)
+    inps : sequence of strings
+        The command as typed by the user, including newlines
+    tss : sequence of two-tuples of floats
+        The timestamps of when the command started and finished, including
+        fractions
+    outs : sequence of strings
+        The output of the command, if xonsh is configured to save it
+
+    In all of these sequences, index 0 is the oldest and -1 (the last item) is the newest.
+    """
 
     def __init__(self, filename=None, sessionid=None, buffersize=100, gc=True,
                  **meta):
@@ -568,8 +535,8 @@ class History(object):
             self.filename = filename
         self.buffer = []
         self.buffersize = buffersize
-        self._queue = deque()
-        self._cond = Condition()
+        self._queue = collections.deque()
+        self._cond = threading.Condition()
         self._len = 0
         self.last_cmd_out = None
         self.last_cmd_rtn = None
@@ -639,30 +606,26 @@ class History(object):
         return hf
 
     def show(self, *args, **kwargs):
-        """
-        Returns shell history as a list
+        """Return shell history as a list
 
         Valid options:
             `session` - returns xonsh history from current session
-            `show`    - alias of `session`
-            `all`     - returns xonsh history from all sessions
-            `xonsh`   - alias of `all`
+            `xonsh`   - returns xonsh history from all sessions
             `zsh`     - returns all zsh history
             `bash`    - returns all bash history
         """
-        return _show(*args, **kwargs)
+        return list(_hist_get(*args, **kwargs))
 
 
-def _info(ns, hist):
+def _hist_info(ns, hist):
     """Display information about the shell history."""
-    data = OrderedDict()
+    data = collections.OrderedDict()
     data['sessionid'] = str(hist.sessionid)
     data['filename'] = hist.filename
     data['length'] = len(hist)
     data['buffersize'] = hist.buffersize
     data['bufferlength'] = len(hist.buffer)
     if ns.json:
-        import json
         s = json.dumps(data)
         print(s)
     else:
@@ -670,7 +633,7 @@ def _info(ns, hist):
         print('\n'.join(lines))
 
 
-def _gc(ns, hist):
+def _hist_gc(ns, hist):
     """Start and monitor garbage collection of the shell history."""
     hist.gc = gc = HistoryGC(wait_for_shell=False, size=ns.size)
     if ns.blocking:
@@ -678,40 +641,55 @@ def _gc(ns, hist):
             continue
 
 
-_HIST_MAIN_ACTIONS = {
-    'show': _show,
-    'xonsh': _show,
-    'zsh': _show,
-    'bash': _show,
-    'session': _show,
-    'all': _show,
-    'id': lambda ns, hist: print(hist.sessionid),
-    'file': lambda ns, hist: print(hist.filename),
-    'info': _info,
-    'diff': _dh_main_action,
-    'gc': _gc,
+@lazyobject
+def _HIST_SESSIONS():
+    return {'session': _curr_session_parser,
+            'xonsh': _all_xonsh_parser,
+            'all': _all_xonsh_parser,
+            'zsh': _zsh_hist_parser,
+            'bash': _bash_hist_parser}
+
+
+@lazyobject
+def _HIST_MAIN_ACTIONS():
+    return {
+        'show': _hist_show,
+        'id': lambda ns, hist: print(hist.sessionid),
+        'file': lambda ns, hist: print(hist.filename),
+        'info': _hist_info,
+        'diff': _dh_main_action,
+        'gc': _hist_gc,
     }
 
 
-def _hist_main(hist, args):
-    """This implements the history CLI."""
-    if not args or args[0] in ['-r'] or ensure_int_or_slice(args[0]):
-        args.insert(0, 'show')
-    elif args[0] not in list(_HIST_MAIN_ACTIONS) + ['-h', '--help']:
-        print("{} is not a valid input.".format(args[0]),
-              file=sys.stderr)
-        return
-    if (args[0] in ['show', 'xonsh', 'zsh', 'bash', 'session', 'all'] and
-        len(args) > 1 and args[-1].startswith('-') and
-            args[-1][1].isdigit()):
-        args.insert(-1, '--')  # ensure parsing stops before a negative int
-    ns = _hist_create_parser().parse_args(args)
-    if ns.action is None:  # apply default action
-        ns = _hist_create_parser().parse_args(['show'] + args)
-    _HIST_MAIN_ACTIONS[ns.action](ns, hist)
+def _hist_parse_args(args):
+    """Prepare and parse arguments for the history command.
+
+    Add default action for ``history`` and
+    default session for ``history show``.
+    """
+    parser = _hist_create_parser()
+    if not args:
+        args = ['show', 'session']
+    elif args[0] not in _HIST_MAIN_ACTIONS and args[0] not in ('-h', '--help'):
+        args = ['show', 'session'] + args
+    if args[0] == 'show':
+        if not any(a in _HIST_SESSIONS for a in args):
+            args.insert(1, 'session')
+        ns, slices = parser.parse_known_args(args)
+        if slices:
+            if not ns.slices:
+                ns.slices = slices
+            else:
+                ns.slices.extend(slices)
+    else:
+        ns = parser.parse_args(args)
+    return ns
 
 
 def history_main(args=None, stdin=None):
     """This is the history command entry point."""
-    _ = stdin
-    _hist_main(builtins.__xonsh_history__, args)  # pylint: disable=no-member
+    hist = builtins.__xonsh_history__
+    ns = _hist_parse_args(args)
+    if ns:
+        _HIST_MAIN_ACTIONS[ns.action](ns, hist)
