@@ -12,14 +12,13 @@ The following implementations were forked from the IPython project:
 * Copyright (c) 2001, Janko Hauser <jhauser@zscout.de>
 * Copyright (c) 2001, Nathaniel Gray <n8gray@caltech.edu>
 
-
 Example Usage
 -------------
 
 To directly print the representation of an object use `pprint`::
 
-    from pretty import pprint
-    pprint(complex_object)
+    from pretty import pretty_print
+    pretty_pprint(complex_object)
 
 To get a string of the output use `pretty`::
 
@@ -69,40 +68,32 @@ line (if the items are short enough) or aligned with the right edge of the
 opening bracket of `MyList`.
 
 If you just want to indent something you can use the group function
-without open / close parameters.  Yu can also use this code::
+without open / close parameters.  You can also use this code::
 
     with p.indent(2):
         ...
 
-Inheritance diagram:
-
-.. inheritance-diagram:: IPython.lib.pretty
-   :parts: 3
 
 :copyright: 2007 by Armin Ronacher.
             Portions (c) 2009 by Robert Kern.
 :license: BSD License.
 """
-from __future__ import print_function
-from contextlib import contextmanager
+import io
+import re
 import sys
 import types
-import re
 import datetime
-from collections import deque
+import contextlib
+import collections
 
-# from IPython.utils.py3compat import PY3, cast_unicode
-# from IPython.utils.encoding import get_stream_enc
+from xonsh.lazyasd import LazyObject, lazyobject
 
-from io import StringIO
-
-
-__all__ = ['pretty', 'pprint', 'PrettyPrinter', 'RepresentationPrinter',
+__all__ = [
+    'pretty', 'pretty_print', 'PrettyPrinter', 'RepresentationPrinter',
     'for_type', 'for_type_by_name']
 
 
 MAX_SEQ_LENGTH = 1000
-_re_pattern_type = type(re.compile(''))
 
 
 def _safe_getattr(obj, attr, default=None):
@@ -116,14 +107,8 @@ def _safe_getattr(obj, attr, default=None):
     except Exception:
         return default
 
-# if PY3:
-CUnicodeIO = StringIO
-# else:
-    # class CUnicodeIO(StringIO):
-        # """StringIO that casts str to unicode on Python 2"""
-        # def write(self, text):
-            # return super(CUnicodeIO, self).write(
-                # cast_unicode(text, encoding=get_stream_enc(sys.stdout)))
+
+CUnicodeIO = io.StringIO
 
 
 def pretty(obj, verbose=False, max_width=79, newline='\n', max_seq_length=MAX_SEQ_LENGTH):
@@ -131,25 +116,25 @@ def pretty(obj, verbose=False, max_width=79, newline='\n', max_seq_length=MAX_SE
     Pretty print the object's representation.
     """
     stream = CUnicodeIO()
-    printer = RepresentationPrinter(stream, verbose, max_width, newline, max_seq_length)
+    printer = RepresentationPrinter(stream, verbose, max_width, newline, max_seq_length=max_seq_length)
     printer.pretty(obj)
     printer.flush()
     return stream.getvalue()
 
 
-def pprint(obj, verbose=False, max_width=79, newline='\n', max_seq_length=MAX_SEQ_LENGTH):
+def pretty_print(obj, verbose=False, max_width=79, newline='\n', max_seq_length=MAX_SEQ_LENGTH):
     """
-    Like `pretty` but print to stdout.
+    Like pretty() but print to stdout.
     """
-    printer = RepresentationPrinter(sys.stdout, verbose, max_width, newline, max_seq_length)
+    printer = RepresentationPrinter(sys.stdout, verbose, max_width, newline, max_seq_length=max_seq_length)
     printer.pretty(obj)
     printer.flush()
     sys.stdout.write(newline)
     sys.stdout.flush()
 
-class _PrettyPrinterBase(object):
 
-    @contextmanager
+class _PrettyPrinterBase(object):
+    @contextlib.contextmanager
     def indent(self, indent):
         """with statement support for indenting/dedenting."""
         self.indentation += indent
@@ -158,14 +143,14 @@ class _PrettyPrinterBase(object):
         finally:
             self.indentation -= indent
 
-    @contextmanager
-    def group(self, indent=0, gopen='', gclose=''):
+    @contextlib.contextmanager
+    def group(self, indent=0, open='', close=''):
         """like begin_group / end_group but for the with statement."""
-        self.begin_group(indent, gopen)
+        self.begin_group(indent, open)
         try:
             yield
         finally:
-            self.end_group(indent, gclose)
+            self.end_group(indent, close)
 
 
 class PrettyPrinter(_PrettyPrinterBase):
@@ -183,7 +168,7 @@ class PrettyPrinter(_PrettyPrinterBase):
         self.max_seq_length = max_seq_length
         self.output_width = 0
         self.buffer_width = 0
-        self.buffer = deque()
+        self.buffer = collections.deque()
 
         root_group = Group(0)
         self.group_stack = [root_group]
@@ -248,7 +233,7 @@ class PrettyPrinter(_PrettyPrinterBase):
         self.output_width = self.indentation
         self.buffer_width = 0
 
-    def begin_group(self, indent=0, gopen=''):
+    def begin_group(self, indent=0, open=''):
         """
         Begin a group.  If you want support for python < 2.5 which doesn't has
         the with statement this is the preferred way:
@@ -266,8 +251,8 @@ class PrettyPrinter(_PrettyPrinterBase):
         the width of the opening text), the second the opening text.  All
         parameters are optional.
         """
-        if gopen:
-            self.text(gopen)
+        if open:
+            self.text(open)
         group = Group(self.group_stack[-1].depth + 1)
         self.group_stack.append(group)
         self.group_queue.enq(group)
@@ -280,17 +265,17 @@ class PrettyPrinter(_PrettyPrinterBase):
                 self.text(',')
                 self.breakable()
                 self.text('...')
-                raise StopIteration
+                return
             yield idx, x
 
-    def end_group(self, dedent=0, gclose=''):
+    def end_group(self, dedent=0, close=''):
         """End a group. See `begin_group` for more details."""
         self.indentation -= dedent
         group = self.group_stack.pop()
         if not group.breakables:
             self.group_queue.remove(group)
-        if gclose:
-            self.text(gclose)
+        if close:
+            self.text(close)
 
     def flush(self):
         """Flush data that is left in the buffer."""
@@ -335,8 +320,8 @@ class RepresentationPrinter(PrettyPrinter):
     """
 
     def __init__(self, output, verbose=False, max_width=79, newline='\n',
-        singleton_pprinters=None, type_pprinters=None, deferred_pprinters=None,
-        max_seq_length=MAX_SEQ_LENGTH):
+                 singleton_pprinters=None, type_pprinters=None, deferred_pprinters=None,
+                 max_seq_length=MAX_SEQ_LENGTH):
 
         PrettyPrinter.__init__(self, output, max_width, newline, max_seq_length=max_seq_length)
         self.verbose = verbose
@@ -412,13 +397,11 @@ class RepresentationPrinter(PrettyPrinter):
 
 
 class Printable(object):
-
     def output(self, stream, output_width):
         return output_width
 
 
 class Text(Printable):
-
     def __init__(self):
         self.objs = []
         self.width = 0
@@ -434,7 +417,6 @@ class Text(Printable):
 
 
 class Breakable(Printable):
-
     def __init__(self, seq, width, pretty):
         self.obj = seq
         self.width = width
@@ -456,15 +438,13 @@ class Breakable(Printable):
 
 
 class Group(Printable):
-
     def __init__(self, depth):
         self.depth = depth
-        self.breakables = deque()
+        self.breakables = collections.deque()
         self.want_break = False
 
 
 class GroupQueue(object):
-
     def __init__(self, *groups):
         self.queue = []
         for group in groups:
@@ -493,10 +473,14 @@ class GroupQueue(object):
         except ValueError:
             pass
 
-try:
-    _baseclass_reprs = (object.__repr__, types.InstanceType.__repr__)
-except AttributeError:  # Python 3
-    _baseclass_reprs = (object.__repr__,)
+
+@lazyobject
+def _baseclass_reprs():
+    try:
+        br = (object.__repr__, types.InstanceType.__repr__)
+    except AttributeError:  # Python 3
+        br = (object.__repr__,)
+    return br
 
 
 def _default_pprint(obj, p, cycle):
@@ -542,6 +526,7 @@ def _seq_pprinter_factory(start, end, basetype):
     Factory that returns a pprint function useful for sequences.  Used by
     the default pprint for tuples, dicts, and lists.
     """
+
     def inner(obj, p, cycle):
         typ = type(obj)
         if basetype is not None and typ is not basetype and typ.__repr__ != basetype.__repr__:
@@ -561,6 +546,7 @@ def _seq_pprinter_factory(start, end, basetype):
             # Special case for 1-item tuples.
             p.text(',')
         p.end_group(step, end)
+
     return inner
 
 
@@ -568,6 +554,7 @@ def _set_pprinter_factory(start, end, basetype):
     """
     Factory that returns a pprint function useful for sets and frozensets.
     """
+
     def inner(obj, p, cycle):
         typ = type(obj)
         if basetype is not None and typ is not basetype and typ.__repr__ != basetype.__repr__:
@@ -596,6 +583,7 @@ def _set_pprinter_factory(start, end, basetype):
                     p.breakable()
                 p.pretty(x)
             p.end_group(step, end)
+
     return inner
 
 
@@ -604,6 +592,7 @@ def _dict_pprinter_factory(start, end, basetype=None):
     Factory that returns a pprint function used by the default pprint of
     dicts and dict proxies.
     """
+
     def inner(obj, p, cycle):
         typ = type(obj)
         if basetype is not None and typ is not basetype and typ.__repr__ != basetype.__repr__:
@@ -629,6 +618,7 @@ def _dict_pprinter_factory(start, end, basetype=None):
             p.text(': ')
             p.pretty(obj[key])
         p.end_group(1, end)
+
     return inner
 
 
@@ -658,7 +648,7 @@ def _re_pattern_pprint(obj, p, cycle):
         p.breakable()
         done_one = False
         for flag in ('TEMPLATE', 'IGNORECASE', 'LOCALE', 'MULTILINE', 'DOTALL',
-            'UNICODE', 'VERBOSE', 'DEBUG'):
+                     'UNICODE', 'VERBOSE', 'DEBUG'):
             if obj.flags & getattr(re, flag):
                 if done_one:
                     p.text('|')
@@ -678,7 +668,16 @@ def _type_pprint(obj, p, cycle):
         return
 
     mod = _safe_getattr(obj, '__module__', None)
-    name = _safe_getattr(obj, '__qualname__', obj.__name__)
+    try:
+        name = obj.__qualname__
+        if not isinstance(name, str):
+            # This can happen if the type implements __qualname__ as a property
+            # or other descriptor in Python 2.
+            raise Exception("Try __name__")
+    except Exception:
+        name = obj.__name__
+        if not isinstance(name, str):
+            name = '<unknown type>'
 
     if mod in (None, '__builtin__', 'builtins', 'exceptions'):
         p.text(name)
@@ -720,54 +719,58 @@ def _exception_pprint(obj, p, cycle):
     p.end_group(step, ')')
 
 
-#: the exception base
-try:
-    _exception_base = BaseException
-except NameError:
-    _exception_base = Exception
+@lazyobject
+def _type_pprinters():
+    #: printers for builtin types
+    tp = {
+        int: _repr_pprint,
+        float: _repr_pprint,
+        str: _repr_pprint,
+        tuple: _seq_pprinter_factory('(', ')', tuple),
+        list: _seq_pprinter_factory('[', ']', list),
+        dict: _dict_pprinter_factory('{', '}', dict),
+        set: _set_pprinter_factory('{', '}', set),
+        frozenset: _set_pprinter_factory('frozenset({', '})', frozenset),
+        super: _super_pprint,
+        type(re.compile('')): _re_pattern_pprint,
+        type: _type_pprint,
+        types.FunctionType: _function_pprint,
+        types.BuiltinFunctionType: _function_pprint,
+        types.MethodType: _repr_pprint,
+        datetime.datetime: _repr_pprint,
+        datetime.timedelta: _repr_pprint,
+    }
+    #: the exception base
+    try:
+        _exception_base = BaseException
+    except NameError:
+        _exception_base = Exception
+    tp[_exception_base] = _exception_pprint
+    try:
+        tp[types.DictProxyType] = _dict_pprinter_factory('<dictproxy {', '}>')
+        tp[types.ClassType] = _type_pprint
+        tp[types.SliceType] = _repr_pprint
+    except AttributeError:  # Python 3
+        tp[slice] = _repr_pprint
+    try:
+        tp[xrange] = _repr_pprint
+        tp[long] = _repr_pprint
+        tp[unicode] = _repr_pprint
+    except NameError:
+        tp[range] = _repr_pprint
+        tp[bytes] = _repr_pprint
+    return tp
 
 
-#: printers for builtin types
-_type_pprinters = {
-    int:                        _repr_pprint,
-    float:                      _repr_pprint,
-    str:                        _repr_pprint,
-    tuple:                      _seq_pprinter_factory('(', ')', tuple),
-    list:                       _seq_pprinter_factory('[', ']', list),
-    dict:                       _dict_pprinter_factory('{', '}', dict),
-
-    set:                        _set_pprinter_factory('{', '}', set),
-    frozenset:                  _set_pprinter_factory('frozenset({', '})', frozenset),
-    super:                      _super_pprint,
-    _re_pattern_type:           _re_pattern_pprint,
-    type:                       _type_pprint,
-    types.FunctionType:         _function_pprint,
-    types.BuiltinFunctionType:  _function_pprint,
-    types.MethodType:           _repr_pprint,
-
-    datetime.datetime:          _repr_pprint,
-    datetime.timedelta:         _repr_pprint,
-    _exception_base:            _exception_pprint
-}
-
-try:
-    _type_pprinters[types.DictProxyType] = _dict_pprinter_factory('<dictproxy {', '}>')
-    _type_pprinters[types.ClassType] = _type_pprint
-    _type_pprinters[types.SliceType] = _repr_pprint
-except AttributeError:  # Python 3
-    _type_pprinters[slice] = _repr_pprint
-
-try:
-    _type_pprinters[xrange] = _repr_pprint
-    _type_pprinters[long] = _repr_pprint
-    _type_pprinters[unicode] = _repr_pprint
-except NameError:
-    _type_pprinters[range] = _repr_pprint
-    _type_pprinters[bytes] = _repr_pprint
-
 #: printers for types specified by name
-_deferred_type_pprinters = {
-}
+@lazyobject
+def _deferred_type_pprinters():
+    dtp = {}
+    for_type_by_name('collections', 'defaultdict', _defaultdict_pprint, dtp=dtp)
+    for_type_by_name('collections', 'OrderedDict', _ordereddict_pprint, dtp=dtp)
+    for_type_by_name('collections', 'deque', _deque_pprint, dtp=dtp)
+    for_type_by_name('collections', 'Counter', _counter_pprint, dtp=dtp)
+    return dtp
 
 
 def for_type(typ, func):
@@ -780,26 +783,31 @@ def for_type(typ, func):
         _type_pprinters[typ] = func
     return oldfunc
 
-def for_type_by_name(type_module, type_name, func):
+
+def for_type_by_name(type_module, type_name, func, dtp=None):
     """
     Add a pretty printer for a type specified by the module and name of a type
     rather than the type object itself.
     """
+    if dtp is None:
+        dtp = _deferred_type_pprinters
     key = (type_module, type_name)
-    oldfunc = _deferred_type_pprinters.get(key, None)
+    oldfunc = dtp.get(key, None)
     if func is not None:
         # To support easy restoration of old pprinters, we need to ignore Nones.
-        _deferred_type_pprinters[key] = func
+        dtp[key] = func
     return oldfunc
 
 
 #: printers for the default singletons
-_singleton_pprinters = dict.fromkeys(map(id, [None, True, False, Ellipsis,
-                                      NotImplemented]), _repr_pprint)
+_singleton_pprinters = LazyObject(lambda: dict.fromkeys(
+    map(id, [None, True, False, Ellipsis,
+             NotImplemented]), _repr_pprint),
+                                  globals(), '_singleton_pprinters')
 
 
 def _defaultdict_pprint(obj, p, cycle):
-    name = 'defaultdict'
+    name = obj.__class__.__name__
     with p.group(len(name) + 1, name + '(', ')'):
         if cycle:
             p.text('...')
@@ -811,7 +819,7 @@ def _defaultdict_pprint(obj, p, cycle):
 
 
 def _ordereddict_pprint(obj, p, cycle):
-    name = 'OrderedDict'
+    name = obj.__class__.__name__
     with p.group(len(name) + 1, name + '(', ')'):
         if cycle:
             p.text('...')
@@ -820,7 +828,7 @@ def _ordereddict_pprint(obj, p, cycle):
 
 
 def _deque_pprint(obj, p, cycle):
-    name = 'deque'
+    name = obj.__class__.__name__
     with p.group(len(name) + 1, name + '(', ')'):
         if cycle:
             p.text('...')
@@ -829,30 +837,9 @@ def _deque_pprint(obj, p, cycle):
 
 
 def _counter_pprint(obj, p, cycle):
-    name = 'Counter'
+    name = obj.__class__.__name__
     with p.group(len(name) + 1, name + '(', ')'):
         if cycle:
             p.text('...')
         elif len(obj):
             p.pretty(dict(obj))
-
-for_type_by_name('collections', 'defaultdict', _defaultdict_pprint)
-for_type_by_name('collections', 'OrderedDict', _ordereddict_pprint)
-for_type_by_name('collections', 'deque', _deque_pprint)
-for_type_by_name('collections', 'Counter', _counter_pprint)
-
-if __name__ == '__main__':
-    from random import randrange
-
-    class Foo(object):
-        def __init__(self):
-            self.foo = 1
-            self.bar = re.compile(r'\s+')
-            self.blub = dict.fromkeys(range(30), randrange(1, 40))
-            self.hehe = 23424.234234
-            self.list = ["blub", "blah", self]
-
-        def get_foo(self):
-            print("foo")
-
-    pprint(Foo(), verbose=True)
