@@ -2,30 +2,29 @@
 import ast
 import json
 import shutil
+import pprint
 import textwrap
 import builtins
+import argparse
 import functools
 import itertools
-from pprint import pformat
-from argparse import ArgumentParser
-from contextlib import contextmanager
+import contextlib
 
 try:
     import ply
 except ImportError:
     from xonsh import ply
 
+import xonsh.wizard as wiz
 from xonsh import __version__ as XONSH_VERSION
 from xonsh.environ import is_template_string
-from xonsh import platform
-from xonsh.platform import is_readline_available, ptk_version
-from xonsh import tools
-from xonsh.wizard import (Wizard, Pass, Message, Save, Load, YesNo, Input,
-    PromptVisitor, While, StoreNonEmpty, create_truefalse_cond, YN, Unstorable,
-    Question)
+from xonsh.platform import (is_readline_available, ptk_version,
+                            PYTHON_VERSION_INFO, pygments_version, ON_POSIX, ON_LINUX, linux_distro,
+                            ON_DARWIN, ON_WINDOWS, ON_CYGWIN, DEFAULT_ENCODING, githash)
+from xonsh.tools import (to_bool, is_string, print_exception, is_superuser,
+                         color_style_names, print_color, color_style)
 from xonsh.xontribs import xontrib_metadata, find_xontrib
 
-
 HR = "'`-.,_,.-*'`-.,_,.-*'`-.,_,.-*'`-.,_,.-*'`-.,_,.-*'`-.,_,.-*'`-.,_,.-*'"
 WIZARD_HEAD = """
           {{BOLD_WHITE}}Welcome to the xonsh configuration wizard!{{NO_COLOR}}
@@ -83,7 +82,7 @@ values are presented as pretty repr strings of their Python types.
 will accept the default value for that entry.
 """.format(hr=HR)
 
-WIZARD_ENV_QUESTION = "Would you like to set env vars now, " + YN
+WIZARD_ENV_QUESTION = "Would you like to set env vars now, " + wiz.YN
 
 WIZARD_XONTRIB = """
 {hr}
@@ -95,60 +94,59 @@ extensions are called {{BOLD_GREEN}}xontribs{{NO_COLOR}}, or xonsh contributions
 Xontribs are dynamically loadable, either by importing them directly or by
 using the 'xontrib' command. However, you can also configure xonsh to load
 xontribs automatically on startup prior to loading the run control files.
-This allows the xontrib to be used immediately your xonshrc files.
+This allows the xontrib to be used immediately in your xonshrc files.
 
 The following describes all xontribs that have been registered with xonsh.
 These come from users, 3rd party developers, or xonsh itself!
 """.format(hr=HR)
 
-WIZARD_XONTRIB_QUESTION = "Would you like to enable xontribs now, " + YN
-
+WIZARD_XONTRIB_QUESTION = "Would you like to enable xontribs now, " + wiz.YN
 
 WIZARD_TAIL = """
 Thanks for using the xonsh configuration wizard!"""
 
 
-
-def make_fs():
+def make_fs_wiz():
     """Makes the foreign shell part of the wizard."""
-    cond = create_truefalse_cond(prompt='Add a foreign shell, ' + YN)
-    fs = While(cond=cond, body=[
-        Input('shell name (e.g. bash): ', path='/foreign_shells/{idx}/shell'),
-        StoreNonEmpty('interactive shell [bool, default=True]: ',
-                      converter=tools.to_bool,
-                      show_conversion=True,
-                      path='/foreign_shells/{idx}/interactive'),
-        StoreNonEmpty('login shell [bool, default=False]: ',
-                      converter=tools.to_bool,
-                      show_conversion=True,
-                      path='/foreign_shells/{idx}/login'),
-        StoreNonEmpty("env command [str, default='env']: ",
-                      path='/foreign_shells/{idx}/envcmd'),
-        StoreNonEmpty("alias command [str, default='alias']: ",
-                      path='/foreign_shells/{idx}/aliascmd'),
-        StoreNonEmpty(("extra command line arguments [list of str, "
-                       "default=[]]: "),
-                      converter=ast.literal_eval,
-                      show_conversion=True,
-                      path='/foreign_shells/{idx}/extra_args'),
-        StoreNonEmpty('current environment [dict, default=None]: ',
-                      converter=ast.literal_eval,
-                      show_conversion=True,
-                      path='/foreign_shells/{idx}/currenv'),
-        StoreNonEmpty('safely handle exceptions [bool, default=True]: ',
-                      converter=tools.to_bool,
-                      show_conversion=True,
-                      path='/foreign_shells/{idx}/safe'),
-        StoreNonEmpty("pre-command [str, default='']: ",
-                      path='/foreign_shells/{idx}/prevcmd'),
-        StoreNonEmpty("post-command [str, default='']: ",
-                      path='/foreign_shells/{idx}/postcmd'),
-        StoreNonEmpty("foreign function command [str, default=None]: ",
-                      path='/foreign_shells/{idx}/funcscmd'),
-        StoreNonEmpty("source command [str, default=None]: ",
-                      path='/foreign_shells/{idx}/sourcer'),
-        Message(message='')  # inserts a newline
-        ])
+    cond = wiz.create_truefalse_cond(prompt='Add a new foreign shell, ' + wiz.YN)
+    fs = wiz.While(cond=cond, body=[
+        wiz.Input('shell name (e.g. bash): ',
+                  path='/foreign_shells/{idx}/shell'),
+        wiz.StoreNonEmpty('interactive shell [bool, default=True]: ',
+                          converter=to_bool,
+                          show_conversion=True,
+                          path='/foreign_shells/{idx}/interactive'),
+        wiz.StoreNonEmpty('login shell [bool, default=False]: ',
+                          converter=to_bool,
+                          show_conversion=True,
+                          path='/foreign_shells/{idx}/login'),
+        wiz.StoreNonEmpty("env command [str, default='env']: ",
+                          path='/foreign_shells/{idx}/envcmd'),
+        wiz.StoreNonEmpty("alias command [str, default='alias']: ",
+                          path='/foreign_shells/{idx}/aliascmd'),
+        wiz.StoreNonEmpty(("extra command line arguments [list of str, "
+                           "default=[]]: "),
+                          converter=ast.literal_eval,
+                          show_conversion=True,
+                          path='/foreign_shells/{idx}/extra_args'),
+        wiz.StoreNonEmpty('current environment [dict, default=None]: ',
+                          converter=ast.literal_eval,
+                          show_conversion=True,
+                          path='/foreign_shells/{idx}/currenv'),
+        wiz.StoreNonEmpty('safely handle exceptions [bool, default=True]: ',
+                          converter=to_bool,
+                          show_conversion=True,
+                          path='/foreign_shells/{idx}/safe'),
+        wiz.StoreNonEmpty("pre-command [str, default='']: ",
+                          path='/foreign_shells/{idx}/prevcmd'),
+        wiz.StoreNonEmpty("post-command [str, default='']: ",
+                          path='/foreign_shells/{idx}/postcmd'),
+        wiz.StoreNonEmpty("foreign function command [str, default=None]: ",
+                          path='/foreign_shells/{idx}/funcscmd'),
+        wiz.StoreNonEmpty("source command [str, default=None]: ",
+                          path='/foreign_shells/{idx}/sourcer'),
+        wiz.Message(message='Foreign shell added.\n')
+    ])
     return fs
 
 
@@ -159,6 +157,7 @@ def _wrap_paragraphs(text, width=70, **kwargs):
     s = '\n'.join(pars)
     return s
 
+
 ENVVAR_MESSAGE = """
 {{BOLD_CYAN}}${name}{{NO_COLOR}}
 {docstr}
@@ -167,6 +166,7 @@ ENVVAR_MESSAGE = """
 
 ENVVAR_PROMPT = "{BOLD_GREEN}>>>{NO_COLOR} "
 
+
 def make_envvar(name):
     """Makes a StoreNonEmpty node for an environment variable."""
     env = builtins.__xonsh_env__
@@ -177,19 +177,19 @@ def make_envvar(name):
     if '\n' in default:
         default = '\n' + _wrap_paragraphs(default, width=69)
     curr = env.get(name)
-    if tools.is_string(curr) and is_template_string(curr):
+    if is_string(curr) and is_template_string(curr):
         curr = curr.replace('{', '{{').replace('}', '}}')
-    curr = pformat(curr, width=69)
+    curr = pprint.pformat(curr, width=69)
     if '\n' in curr:
         curr = '\n' + curr
     msg = ENVVAR_MESSAGE.format(name=name, default=default, current=curr,
                                 docstr=_wrap_paragraphs(vd.docstr, width=69))
-    mnode = Message(message=msg)
+    mnode = wiz.Message(message=msg)
     ens = env.get_ensurer(name)
     path = '/env/' + name
-    pnode = StoreNonEmpty(ENVVAR_PROMPT, converter=ens.convert,
-                          show_conversion=True, path=path, retry=True,
-                          store_raw=vd.store_as_str)
+    pnode = wiz.StoreNonEmpty(ENVVAR_PROMPT, converter=ens.convert,
+                              show_conversion=True, path=path, retry=True,
+                              store_raw=vd.store_as_str)
     return mnode, pnode
 
 
@@ -200,17 +200,18 @@ def _make_flat_wiz(kidfunc, *args):
         if k is None:
             continue
         flatkids.extend(k)
-    wiz = Wizard(children=flatkids)
-    return wiz
+    wizard = wiz.Wizard(children=flatkids)
+    return wizard
 
 
-def make_env():
+def make_env_wiz():
     """Makes an environment variable wizard."""
-    w = _make_flat_wiz(make_envvar, sorted(builtins.__xonsh_env__.docs.keys()))
+    w = _make_flat_wiz(make_envvar, sorted(builtins.__xonsh_env__._docs.keys()))
     return w
 
 
-XONTRIB_PROMPT = '{BOLD_GREEN}Add this xontrib{NO_COLOR}, ' + YN
+XONTRIB_PROMPT = '{BOLD_GREEN}Add this xontrib{NO_COLOR}, ' + wiz.YN
+
 
 def _xontrib_path(visitor=None, node=None, val=None):
     # need this to append only based on user-selected size
@@ -238,14 +239,14 @@ def make_xontrib(xontrib, package):
     msg += _wrap_paragraphs(desc, width=69)
     if msg.endswith('\n'):
         msg = msg[:-1]
-    mnode = Message(message=msg)
-    convert = lambda x: name if tools.to_bool(x) else Unstorable
-    pnode = StoreNonEmpty(XONTRIB_PROMPT, converter=convert,
-                          path=_xontrib_path)
+    mnode = wiz.Message(message=msg)
+    convert = lambda x: name if to_bool(x) else wiz.Unstorable
+    pnode = wiz.StoreNonEmpty(XONTRIB_PROMPT, converter=convert,
+                              path=_xontrib_path)
     return mnode, pnode
 
 
-def make_xontribs():
+def make_xontribs_wiz():
     """Makes a xontrib wizard."""
     md = xontrib_metadata()
     pkgs = [md['packages'].get(d.get('package', None), {}) for d in md['xontribs']]
@@ -253,7 +254,7 @@ def make_xontribs():
     return w
 
 
-def make_wizard(default_file=None, confirm=False):
+def make_xonfig_wizard(default_file=None, confirm=False):
     """Makes a configuration wizard for xonsh config file.
 
     Parameters
@@ -263,38 +264,44 @@ def make_wizard(default_file=None, confirm=False):
     confirm : bool, optional
         Confirm that the main part of the wizard should be run.
     """
-    wiz = Wizard(children=[
-            Message(message=WIZARD_HEAD),
-            Load(default_file=default_file, check=True),
-            Message(message=WIZARD_FS),
-            make_fs(),
-            Message(message=WIZARD_ENV),
-            YesNo(question=WIZARD_ENV_QUESTION, yes=make_env(), no=Pass()),
-            Message(message=WIZARD_XONTRIB),
-            YesNo(question=WIZARD_XONTRIB_QUESTION, yes=make_xontribs(), no=Pass()),
-            Message(message='\n' + HR + '\n'),
-            Save(default_file=default_file, check=True),
-            Message(message=WIZARD_TAIL),
-            ])
+    w = wiz.Wizard(children=[
+        wiz.Message(message=WIZARD_HEAD),
+        wiz.Load(default_file=default_file, check=True),
+        wiz.Message(message=WIZARD_FS),
+        make_fs_wiz(),
+        wiz.Message(message=WIZARD_ENV),
+        wiz.YesNo(question=WIZARD_ENV_QUESTION, yes=make_env_wiz(),
+                  no=wiz.Pass()),
+        wiz.Message(message=WIZARD_XONTRIB),
+        wiz.YesNo(question=WIZARD_XONTRIB_QUESTION, yes=make_xontribs_wiz(),
+                  no=wiz.Pass()),
+        wiz.Message(message='\n' + HR + '\n'),
+        wiz.Save(default_file=default_file, check=True),
+        wiz.Message(message=WIZARD_TAIL),
+    ])
     if confirm:
         q = ("Would you like to run the xonsh configuration wizard now?\n\n"
-             "1. Yes\n2. No, but ask me later.\n3. No, and don't ask me again."
-             "\n\n1, 2, or 3 [default: 2]? ")
-        passer = Pass()
-        saver = Save(check=False, ask_filename=False, default_file=default_file)
-        wiz = Question(q, {1: wiz, 2: passer, 3: saver},
-                       converter=lambda x: int(x) if x != '' else 2)
-    return wiz
+             "1. Yes (You can abort at any time)\n"
+             "2. No, but ask me next time.\n"
+             "3. No, and don't ask me again.\n\n"
+             "1, 2, or 3 [default: 2]? ")
+        passer = wiz.Pass()
+        saver = wiz.Save(check=False, ask_filename=False,
+                         default_file=default_file)
+        w = wiz.Question(q, {1: w, 2: passer, 3: saver},
+                         converter=lambda x: int(x) if x != '' else 2)
+    return w
 
 
 def _wizard(ns):
     env = builtins.__xonsh_env__
     shell = builtins.__xonsh_shell__.shell
     fname = env.get('XONSHCONFIG') if ns.file is None else ns.file
-    wiz = make_wizard(default_file=fname, confirm=ns.confirm)
+    w = make_xonfig_wizard(default_file=fname, confirm=ns.confirm)
     tempenv = {'PROMPT': '', 'XONSH_STORE_STDOUT': False}
-    pv = PromptVisitor(wiz, store_in_history=False, multiline=False)
-    @contextmanager
+    pv = wiz.PromptVisitor(w, store_in_history=False, multiline=False)
+
+    @contextlib.contextmanager
     def force_hide():
         if env.get('XONSH_STORE_STDOUT') and hasattr(shell, '_force_hide'):
             orig, shell._force_hide = shell._force_hide, False
@@ -302,19 +309,20 @@ def _wizard(ns):
             shell._force_hide = orig
         else:
             yield
+
     with force_hide(), env.swap(tempenv):
         try:
             pv.visit()
         except (KeyboardInterrupt, Exception):
-            tools.print_exception()
+            print_exception()
 
 
-def _format_human(data):
+def _xonfig_format_human(data):
     wcol1 = wcol2 = 0
     for key, val in data:
         wcol1 = max(wcol1, len(key))
         wcol2 = max(wcol2, len(str(val)))
-    hr = '+' + ('-'*(wcol1+2)) + '+' + ('-'*(wcol2+2)) + '+\n'
+    hr = '+' + ('-' * (wcol1 + 2)) + '+' + ('-' * (wcol2 + 2)) + '+\n'
     row = '| {key!s:<{wcol1}} | {val!s:<{wcol2}} |\n'
     s = hr
     for key, val in data:
@@ -323,33 +331,41 @@ def _format_human(data):
     return s
 
 
-def _format_json(data):
+def _xonfig_format_json(data):
     data = {k.replace(' ', '_'): v for k, v in data}
     s = json.dumps(data, sort_keys=True, indent=1) + '\n'
     return s
 
 
 def _info(ns):
+    env = builtins.__xonsh_env__
+    try:
+        ply.__version__ = ply.__version__
+    except AttributeError:
+        ply.__version__ = '3.8'
     data = [
         ('xonsh', XONSH_VERSION),
-        ('Python', '{}.{}.{}'.format(*platform.PYTHON_VERSION_INFO)),
+        ('Git SHA', githash()),
+        ('Python', '{}.{}.{}'.format(*PYTHON_VERSION_INFO)),
         ('PLY', ply.__version__),
         ('have readline', is_readline_available()),
         ('prompt toolkit', ptk_version() or None),
-        ('shell type', builtins.__xonsh_env__.get('SHELL_TYPE')),
-        ('pygments', platform.PYGMENTS_VERSION),
-        ('on posix', platform.ON_POSIX),
-        ('on linux', platform.ON_LINUX)]
-    if platform.ON_LINUX:
-        data.append(('distro', platform.LINUX_DISTRO))
+        ('shell type', env.get('SHELL_TYPE')),
+        ('pygments', pygments_version()),
+        ('on posix', bool(ON_POSIX)),
+        ('on linux', ON_LINUX)]
+    if ON_LINUX:
+        data.append(('distro', linux_distro()))
     data.extend([
-        ('on darwin', platform.ON_DARWIN),
-        ('on windows', platform.ON_WINDOWS),
-        ('on cygwin', platform.ON_CYGWIN),
-        ('is superuser', tools.IS_SUPERUSER),
-        ('default encoding', platform.DEFAULT_ENCODING),
-        ])
-    formatter = _format_json if ns.json else _format_human
+        ('on darwin', ON_DARWIN),
+        ('on windows', ON_WINDOWS),
+        ('on cygwin', ON_CYGWIN),
+        ('is superuser', is_superuser()),
+        ('default encoding', DEFAULT_ENCODING),
+        ('xonsh encoding', env.get('XONSH_ENCODING')),
+        ('encoding errors', env.get('XONSH_ENCODING_ERRORS')),
+    ])
+    formatter = _xonfig_format_json if ns.json else _xonfig_format_human
     s = formatter(data)
     return s
 
@@ -357,7 +373,7 @@ def _info(ns):
 def _styles(ns):
     env = builtins.__xonsh_env__
     curr = env.get('XONSH_COLOR_STYLE')
-    styles = sorted(tools.color_style_names())
+    styles = sorted(color_style_names())
     if ns.json:
         s = json.dumps(styles, sort_keys=True, indent=1)
         print(s)
@@ -369,7 +385,7 @@ def _styles(ns):
         else:
             lines.append('  ' + style)
     s = '\n'.join(lines)
-    tools.print_color(s)
+    print_color(s)
 
 
 def _str_colors(cmap, cols):
@@ -382,13 +398,14 @@ def _str_colors(cmap, cols):
         for i, name in enumerate(group):
             buf = ' ' * (width - len(name))
             line += '{' + name + '}' + name + '{NO_COLOR}' + buf
-            if (i+1)%n == 0:
+            if (i + 1) % n == 0:
                 lines.append(line)
                 line = ''
         if len(line) != 0:
             lines.append(line)
     return '\n'.join(lines)
 
+
 def _tok_colors(cmap, cols):
     from xonsh.pyghooks import Color
     nc = Color.NO_COLOR
@@ -406,67 +423,65 @@ def _tok_colors(cmap, cols):
         for i, name in enumerate(group):
             toks.append((names_toks[name], name))
             buf = ' ' * (width - len(name))
-            if (i+1)%n == 0:
+            if (i + 1) % n == 0:
                 buf += '\n'
             toks.append((nc, buf))
         if not toks[-1][1].endswith('\n'):
             toks[-1] = (nc, toks[-1][1] + '\n')
     return toks
 
+
 def _colors(ns):
     cols, _ = shutil.get_terminal_size()
-    if tools.ON_WINDOWS:
+    if ON_WINDOWS:
         cols -= 1
-    cmap = tools.color_style()
+    cmap = color_style()
     akey = next(iter(cmap))
     if isinstance(akey, str):
         s = _str_colors(cmap, cols)
     else:
         s = _tok_colors(cmap, cols)
-    tools.print_color(s)
+    print_color(s)
 
 
-@functools.lru_cache()
-def _create_parser():
-    p = ArgumentParser(prog='xonfig',
-                       description='Manages xonsh configuration.')
+@functools.lru_cache(1)
+def _xonfig_create_parser():
+    p = argparse.ArgumentParser(prog='xonfig',
+                                description='Manages xonsh configuration.')
     subp = p.add_subparsers(title='action', dest='action')
     info = subp.add_parser('info', help=('displays configuration information, '
                                          'default action'))
     info.add_argument('--json', action='store_true', default=False,
                       help='reports results as json')
     wiz = subp.add_parser('wizard', help=('displays configuration information, '
-                                         'default action'))
+                                          'default action'))
     wiz.add_argument('--file', default=None,
                      help='config file location, default=$XONSHCONFIG')
     wiz.add_argument('--confirm', action='store_true', default=False,
-                      help='confirm that the wizard should be run.')
+                     help='confirm that the wizard should be run.')
     sty = subp.add_parser('styles', help='prints available xonsh color styles')
     sty.add_argument('--json', action='store_true', default=False,
                      help='reports results as json')
-    clrs = subp.add_parser('colors', help=('displays the color palette for '
-                                           'the current xonsh color style'))
+    subp.add_parser('colors', help=('displays the color palette for '
+                                    'the current xonsh color style'))
     return p
 
 
-_MAIN_ACTIONS = {
+_XONFIG_MAIN_ACTIONS = {
     'info': _info,
     'wizard': _wizard,
     'styles': _styles,
     'colors': _colors,
-    }
+}
 
-def main(args=None):
+
+def xonfig_main(args=None):
     """Main xonfig entry point."""
-    if not args or (args[0] not in _MAIN_ACTIONS and
+    if not args or (args[0] not in _XONFIG_MAIN_ACTIONS and
                     args[0] not in {'-h', '--help'}):
         args.insert(0, 'info')
-    parser = _create_parser()
+    parser = _xonfig_create_parser()
     ns = parser.parse_args(args)
     if ns.action is None:  # apply default action
         ns = parser.parse_args(['info'] + args)
-    return _MAIN_ACTIONS[ns.action](ns)
-
-
-if __name__ == '__main__':
-    main()
+    return _XONFIG_MAIN_ACTIONS[ns.action](ns)
